
from qserious import deserialize

result_1 = {'hola':'caracola','adios':'caracol','foo':'bar','nested':{'value': 'foobar'}}

def test_dots ():
    assert deserialize(r'hola=caracola&adios=caracol&foo=bar&nested.value=foobar') == result_1

def test_brackets ():
    assert deserialize(r'hola=caracola&adios=caracol&foo=bar&nested[value]=foobar') == result_1

def test_brackets_start ():
    assert deserialize(r'hola=caracola&adios=caracol&foo=bar&[nested]value=foobar') == result_1

def test_url ():
    assert deserialize(r'origin[url]=https%3A%2F%2Fwww.example.com%2Fitem%2Fitem_id') == {'origin':{'url':'https://www.example.com/item/item_id'}}




from qserious import serialize, deserialize

data_1 = {'hola':'caracola','adios':'caracol','foo':'bar','nested':{'value': 'foobar'}}

def test_data1_1 ():
    assert serialize(data_1) == 'nested[value]=foobar&foo=bar&hola=caracola&adios=caracol'

def test_data1_2 ():
    assert deserialize(serialize(data_1)) == data_1

def test_data2 ():
    assert serialize({'nested':{'value': 'foobar'}}) == 'nested[value]=foobar'



# QueryString deserializer

import re
import urllib

def setKey (o, keys, value):
    if( len(keys) > 1 ):
        o[keys[0]] = {}
        setKey(o[keys[0]], keys[1:], value)
    else:
        o[keys[0]] = value

def bracketsToDots (text):
    return re.sub(r"\]\[|\[|\]", ".", re.sub(r"\]$", "", re.sub(r"^\[", "", text)) )

def deserialize(qs):
    if( not isinstance(qs, str) ):
        raise TypeError('should be a string')

    params = qs.split('&')
    result = {}

    for param in params:
        parts = param.split('=')
        setKey(result, bracketsToDots(parts[0]).split('.'), urllib.unquote(parts[1]) )

    return result

def keysTobrackets (keys):
    result = ''
    for i, key in enumerate(keys):
        if i > 0:
            result += '[' + key + ']'
        else:
            result += key
    return result

def _serialize (data, params, keys):
    iterable = False
    try:
        iter(data)
        iterable = True
    except:
        pass

    if not iterable or isinstance(data, str):
        params.append( keysTobrackets(keys) + '=' + urllib.quote( str(data), safe='~()*!.\'') )
        return

    for k in data:
        _keys = keys[:]
        _keys.append(k)
        _serialize( data[k], params, _keys )

def serialize (data):
    params = []
    _serialize(data, params, [])
    return '&'.join(params)



# -*- coding: utf-8 -*-
"""
    qserious
    ~~~~~
"""

from .qserious import deserialize, serialize, setKey, bracketsToDots, keysTobrackets

__version__ = '0.2.8'

# __all__ = ['deserialize', 'serialize', 'setKey', 'bracketsToDots', 'keysTobrackets']




from qserious import keysTobrackets

def test_brackets_2():
    assert keysTobrackets(['foo', 'bar']) == "foo[bar]"

def test_brackets_3():
    assert keysTobrackets(['foo', 'bar', 'foobar']) == "foo[bar][foobar]"



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

import io
import re

with io.open('qserious/__init__.py', 'rt', encoding='utf8') as f:
    version = re.search(r'__version__ = \'(.*?)\'', f.read()).group(1)

setup(
  name = 'qserious',
  packages=find_packages(exclude=['contrib', 'docs', 'tests/*']),
  # packages=['qserious'],
  version = version,
  description = 'query string serializer/deserializer',
  author = 'Jesus Germade',
  author_email = 'jesus@germade.es',
  url = 'https://github.com/pyKilt/qserious', # use the URL to the github repo
  # download_url = 'https://github.com/peterldowns/mypackage/archive/0.1.tar.gz', # I'll explain this in a second
  keywords = ['querystring', 'query', 'string', 'serializer', 'deserializer', 'serialize', 'deserialize'], # arbitrary keywords
  classifiers = [],
  license='MIT',
)




from qserious import bracketsToDots

def test_brackets_1():
    assert bracketsToDots("[foo]bar") == "foo.bar"
    assert bracketsToDots("[foo][bar]") == "foo.bar"
    assert bracketsToDots("foo[bar]") == "foo.bar"

def test_brackets_2():
    assert bracketsToDots("foo[bar][foobar]") == "foo.bar.foobar"
    assert bracketsToDots("[foo][bar]foobar") == "foo.bar.foobar"
